@@ -150,12 +150,24 @@ class PlotManager(QObject):
 
         self.cur = None
 
+        self.legend_loc = 'upper left'
+        self.legend_fontsize = 8
+
     def set_dataframe(self,df):
         # package function set_dataframe checks that the index is datetime index
+
+        # clear the _taginfo to avoid unnecesary looping in clear_all_plots
+        self._taginfo.clear()
         self.clear_all_plots()
 
-        self._df = df
-        self._taginfo.clear()
+        # Check if there are duplicated columns:
+        dupcols = df.columns[ df.columns.duplicated() ]
+        if len(dupcols) > 0:
+            sys.stderr.write('WARNING: Dataframe has duplicated columns, duplicates are being dropped.\n')
+            for c in dupcols:
+                sys.stderr.write('  Dropping {}\n'.format(c))
+
+        self._df = df.drop(columns=dupcols)
 
         for tag in self._df:
             # Check if we can plot the tag
@@ -207,11 +219,18 @@ class PlotManager(QObject):
             print('PlotManager::clear_all_plots()')
 
         try:
-            
+          
+            # Clear plot info for all tags (not just the ones that are known to
+            # be plotted
+            for t in self._taginfo:
+                self._taginfo[t].plotinfo = None
+           
+            '''
             while len(self._plotinfo) > 0:
                 p = self._plotinfo.pop()
                 for t in p.tagnames:
                     self._taginfo[t].plotinfo = None
+            '''
             
             self._plotinfo.clear()
             self._groupid_plots.clear()
@@ -259,7 +278,8 @@ class PlotManager(QObject):
                 color=self._taginfo[tagname].color,
                 label=tagname,
             )
-        plotinfo.ax.legend()
+        plotinfo.ax.legend(loc=self.legend_loc,
+                           fontsize=self.legend_fontsize)
 
         if save_xlim:
             plotinfo.ax.set_xlim(xlim)
@@ -296,7 +316,8 @@ class PlotManager(QObject):
                 label=tag,
                 scalex=False,
             )
-            plotinfo.ax.legend()
+            plotinfo.ax.legend(loc=self.legend_loc,
+                               fontsize=self.legend_fontsize)
             '''
             self._df[tag].plot(color=taginfo.color,
                                ax=plotinfo.ax,
@@ -390,10 +411,11 @@ class PlotManager(QObject):
                 del self._groupid_plots[taginfo.groupid] 
 
             nplots = len(self._plotinfo)
-            gs = matplotlib.gridspec.GridSpec(nplots,1)
-            for i in range(nplots):
-                self._plotinfo[i].ax.set_position( gs[i].get_position(self.plot_window.fig) )
-                self._plotinfo[i].ax.set_subplotspec( gs[i] )
+            if nplots > 0:
+                gs = matplotlib.gridspec.GridSpec(nplots,1)
+                for i in range(nplots):
+                    self._plotinfo[i].ax.set_position( gs[i].get_position(self.plot_window.fig) )
+                    self._plotinfo[i].ax.set_subplotspec( gs[i] )
 
             if len(self._plotinfo) == 0:
                 if DEBUG:
@@ -453,6 +475,20 @@ class PlotManager(QObject):
         code = ''
 
         if nrows > 0:
+            # Create a copy of the dataframe with only displayed data
+            # instead of setting xlimits. to plot faster.
+
+            # xlimit based on first axes, the rest should be the same
+            # this code is only executed if there is at least one plot
+               
+            if ( type(self._df.index) != pandas.DatetimeIndex):
+                code += "df_plot = df\n"
+            else:
+                xlim = self._plotinfo[0].ax.get_xlim()
+                x0 = matplotlib.dates.num2date(xlim[0]).strftime('%Y-%m-%d %H:%M')
+                x1 = matplotlib.dates.num2date(xlim[1]).strftime('%Y-%m-%d %H:%M')
+                code += "df_plot = df['" + x0 + "':'" + x1 + "']\n"
+
             code += 'fig,ax = plt.subplots(nrows={},sharex=True)\n'.format(nrows)
 
             i = 0 
@@ -461,16 +497,11 @@ class PlotManager(QObject):
                 for tag in plotinfo.tagnames:
                     color.append( self._taginfo[tag].color )
 
-                xlim = plotinfo.ax.get_xlim()
-                x0 = matplotlib.dates.num2date(xlim[0])
-                x1 = matplotlib.dates.num2date(xlim[1])
                 ylim = plotinfo.ax.get_ylim()
 
-                code += 'df.plot(\n' + \
+                code += 'df_plot.plot(\n' + \
                         '    y={},\n'.format(plotinfo.tagnames) + \
                         '    color={},\n'.format(color) + \
-                        '    xlim=("' + x0.strftime('%Y-%m-%d %H:%M') +'",\n' + \
-                        '          "' + x1.strftime('%Y-%m-%d %H:%M') + '"),\n' + \
                         '    ylim={},\n'.format(ylim)
 
                 if nrows > 1:
@@ -482,7 +513,17 @@ class PlotManager(QObject):
 
                 i += 1
 
+            if type(self.legend_loc) == str:
+                loccode = "loc='{}'".format(self.legend_loc)
+            else:
+                loccode = "loc={}".format(self.legend_loc)
+            if type(self.legend_fontsize) == str:
+                fontcode = "fontsize='{}'".format(self.legend_fontsize)
+            else:
+                fontcode = "fontsize={}".format(self.legend_fontsize)
 
+            code += "for a in ax:\n    a.legend({},{})\n".format(
+                loccode,fontcode)
             code += 'fig.tight_layout()\n'
 
         #print(code)
@@ -536,8 +577,8 @@ class PlotWindow(QWidget):
 
     @QtCore.pyqtSlot()
     def home_clicked(self):
-        if self.toolbar._nav_stack.empty():
-            self.home_zoom_signal.emit()
+        self.home_zoom_signal.emit()
+        self.toolbar._nav_stack.clear()
                 
 
 
@@ -706,7 +747,7 @@ class TagTool(QWidget):
         self.plot_button.setChecked(False)
         self.blockSignals(False)
 
-def add_grouping_rule(expr,color=None,sub=r'\1'):
+def add_grouping_rule(expr,color=None,sub=r'\1',top=True):
     '''
     Add a rule to group trends.
 
@@ -786,12 +827,20 @@ def add_grouping_rule(expr,color=None,sub=r'\1'):
         regular expression replacement str to return groupid.  Default is r'\\1'
         which returns the first group in expr.  If set to None, then a groupid
         of None is returned (tag is ungrouped).
+    top : bool, optional
+        Set to false to add rule to bottom of rule list.  Default is to add
+        rules to bottom of rule list, the first rule that evaluates is used.
 
     '''
 
-    TagInfo.taginfo_rules.insert(0,
-        TagInfoRule(expr,color,sub)
-    )
+    if top:
+        TagInfo.taginfo_rules.insert(0,
+            TagInfoRule(expr,color,sub)
+        )
+    else:
+        TagInfo.taginfo_rules.append(
+            TagInfoRule(expr,color,sub)
+        )
 
 def remove_grouping_rules(index=None):
     '''
@@ -845,9 +894,9 @@ def load_grouping_template(template):
     ProfCon : Honeywell Profit Controller history
         - Groups .READVALUE, .HIGHLIMIT, .LOWLIMIT, .SSVALUE, .UNBIASEDMODELPV
           per tag
-        - Groups .CONSTRAINTTYPE for MVs and CVs
+        - Groups .CONSTRAINTTYPE and .STATUS for MVs and CVs
     DMC : Aspentech DMC plus history
-        - Groups .ULINMD .LLINDM .VIND .SSIND .LDEPTG .UDEPTG .DEP .SSDEP per tag
+        - Groups .ULINMD .LLINDM .VIND .SSMAN .LDEPTG .UDEPTG .SSDEP per tag
         - Groups .SRVDEP for all tags
         - Groups .SRIIND for all tags
         - Groups .CSIDEP for all tags
@@ -875,27 +924,76 @@ def load_grouping_template(template):
         add_grouping_rule(r'(.*)\.SSVALUE','cyan')
         add_grouping_rule(r'(.*)\.UNBIASEDMODELPV','purple')
         add_grouping_rule(r'(.*)(CV|MV)[0-9]{1,2}\.CONSTRAINTTYPE',sub=r'\2_CONSTRAINTTYPE')
+        add_grouping_rule(r'(.*)(CV|MV)[0-9]{1,2}\.STATUS',sub=r'\2_STATUS')
     elif template == 'DMC':
-        add_grouping_rule(r'(.*)\.VIND','C0')
+        # DMC has catch-all at bottom of list because .VIND and .DEP are not marked.
+        add_grouping_rule(r'(.*)','C0',top=False)
+
+        # MV Parameters
         add_grouping_rule(r'(.*)\.ULINDM','red')
         add_grouping_rule(r'(.*)\.LLINDM','red')
-        add_grouping_rule(r'(.*)\.SSIND','cyan')
+        add_grouping_rule(r'(.*)\.SSMAN','cyan')
+        add_grouping_rule(r'(.*)\.ETMV','lightgreen')
+        add_grouping_rule(r'(.*)\.VINDSP','gray')
 
-        add_grouping_rule(r'(.*)\.DEP','C0')
+        # CV Parameters
         add_grouping_rule(r'(.*)\.UDEPTG','red')
         add_grouping_rule(r'(.*)\.LDEPTG','red')
         add_grouping_rule(r'(.*)\.SSDEP','cyan')
+        add_grouping_rule(r'(.*)\.ETCV','lightgreen')
+        add_grouping_rule(r'(.*)\.PRDMDLD','magenta')
 
+        # Ramp Parameters
+        add_grouping_rule(r'(.*).LRDPTG','red',r'\1_RAMP')
+        add_grouping_rule(r'(.*).URDPTG','red',r'\1_RAMP')
+        add_grouping_rule(r'(.*).SSRDEP','cyan',r'\1_RAMP')
+        add_grouping_rule(r'(.*).RAMPSP','yellow')
+
+        # Generic Variable parameters: e.g. plot all CV statuses on the same
+        # trend
         add_grouping_rule(r'(.*)\.SRVDEP',sub='SRVDEP')
         add_grouping_rule(r'(.*)\.SRIIND',sub='SRIIND')
         add_grouping_rule(r'(.*)\.CSIDEP',sub='CSIDEP')
         add_grouping_rule(r'(.*)\.CSIIND',sub='CSIIND')
+
         
     else:
         print("Unknown template {}".format(template))
 
         
-        
+def set_legend_fontsize(size):
+    '''
+    Parameters:
+    -----------
+    size : int or {'xx-small', 'x-small', 'small', 'medium', 'large', 'x-large', 'xx-large'}
+        font size that can be passed to a matplotlib axes.legend function
+
+    '''
+    plot_manager.legend_fontsize = size
+def set_legend_loc(loc):
+    '''
+    Parameters:
+    -----------
+    loc : int or string
+        legend location that can be passed to matplotlib axes.legend function.
+            ===============   =============
+            Location String   Location Code
+            ===============   =============
+            'best'            0
+            'upper right'     1
+            'upper left'      2
+            'lower left'      3
+            'lower right'     4
+            'right'           5
+            'center left'     6
+            'center right'    7
+            'lower center'    8
+            'upper center'    9
+            'center'          10
+            ===============   =============
+    '''
+    plot_manager.legend_loc = loc
+
 def set_dataframe(df):
     '''
     Set the dataframe to use for plotting.
